#include "datatypes.h"

#include "algorithms.h"


//////// Fielding ////////

Uint32 getLong(BYTE *Message, const Uint32 Offset)
{
	return *(Uint32*)&(Message[Offset]);
}

Uint16 getShort(BYTE *Message, const Uint32 Offset)
{
	return *(Uint16*)&(Message[Offset]);
}

Uint32 getLong(char *Message, const Uint32 Offset)
{
	return *(Uint32*)&(Message[Offset]);
}

Uint16 getShort(char *Message, const Uint32 Offset)
{
	return *(Uint16*)&(Message[Offset]);
}

BYTE getByte(char *Message, const Uint32 Offset)
{
	return *(BYTE*)&(Message[Offset]);
}


//////// Strings ////////

Uint32 STRLEN(char *string)
{
	Uint32 len = 0;

	while (string[len])	++len;

	return len;
}

bool CMPSTART(char *control, char *constant)
{
	char c;

	while (c = *control)
	{
		if (*constant != c)
			return false;

		++control;
		++constant;
	}

	return true;
}

bool CMPSTR(char *a, char *b)
{
	char c;

	while (tolower(c = *a) == tolower(*b))
	{
		if (!c) return true;

		++a;
		++b;
	}

	return false;
}

String::String()
{
	msg = new char[1];
	len = 0;
	*msg = '\0';
}

String::String(char *s)
{
	Uint32 slen = STRLEN(s);

	msg = new char[slen + 1];
	len = slen;
	MEMCPY(s, msg, slen);
	msg[slen] = '\0';
}

String::String(const Uint32 number)
{
	String s = getString(number, 10, 0, true);
	Uint32 slen = s.len;

	msg = new char[slen + 1];
	len = slen;
	MEMCPY(s.msg, msg, slen);
	msg[slen] = '\0';
}

String::String(const String &s)
{
	Uint32 slen = s.len;

	msg = new char[slen + 1];
	len = slen;
	MEMCPY(s.msg, msg, slen);
	msg[slen] = '\0';
}

String::~String()
{
	delete []msg;
	msg = NULL;
}

void String::append(char *buffer, Uint32 nlen)
{
	Uint32 clen = nlen + len;

	char *temp = new char[clen + 1];
	MEMCPY(msg, temp, len);
	MEMCPY(buffer, temp + len, nlen);
	temp[clen] = '\0';

	delete []msg;
	msg = temp;
	len = clen;
}

void String::prepend(char *buffer, Uint32 nlen)
{
	Uint32 clen = nlen + len;

	char *temp = new char[clen + 1];
	MEMCPY(buffer, temp, nlen);
	MEMCPY(msg, temp + nlen, len);
	temp[clen] = '\0';

	delete []msg;
	msg = temp;
	len = clen;
}

void String::set(char *buffer, Uint32 nlen)
{
	delete []msg;

	msg = new char[nlen + 1];
	len = nlen;
	MEMCPY(buffer, msg, nlen);
	msg[nlen] = '\0';
}

void String::clear()
{
	delete []msg;

	msg = new char[1];
	len = 0;
	*msg = '\0';
}

bool String::operator==(char *string)
{
	return CMPSTR(string, msg);
}

bool String::operator==(Uint32 number)
{
	String s = number;

	return CMPSTR(s.msg, msg);
}

bool String::operator==(String &string)
{
	return CMPSTR(string.msg, msg);
}

bool String::operator!=(char *string)
{
	return !CMPSTR(string, msg);
}

bool String::operator!=(Uint32 number)
{
	String s = number;

	return !CMPSTR(s.msg, msg);
}

bool String::operator!=(String &string)
{
	return !CMPSTR(string.msg, msg);
}

void String::operator=(char *string)
{
	set(string, STRLEN(string));
}

void String::operator=(Uint32 number)
{
	String s = number;

	set(s.msg, s.len);
}

void String::operator=(String &string)
{
	set(string.msg, string.len);
}

void String::operator+=(char *string)
{
	append(string, STRLEN(string));
}

void String::operator+=(Uint32 number)
{
	String s = number;

	append(s.msg, s.len);
}

void String::operator+=(String &string)
{
	append(string.msg, string.len);
}

String String::operator+(char *string)
{
	String s(*this);
	s.append(string, STRLEN(string));

	return s;
}

String String::operator+(Uint32 number)
{
	String n(number);

	String s(*this);
	s.append(n.msg, n.len);

	return s;
}

String String::operator+(String &string)
{
	String s(*this);
	s.append(string.msg, string.len);

	return s;
}

String operator+(char *prepended, String &string)
{
	String s(prepended);
	s.append(string.msg, string.len);

	return s;
}

String::operator char*()
{
	return msg;
}

String String::left(Uint32 nlen)
{
	String s;

	nlen = limit(nlen, len);

	s.set(msg, nlen);

	return s;
}

String String::right(Uint32 nlen)
{
	String s;

	nlen = limit(nlen, len);

	s.set(msg + (len - nlen), nlen);

	return s;
}

String String::mid(Uint32 offset, Uint32 nlen = 32000)
{
	String s;

	if (offset >= len) return s;

	if (offset + nlen > len) nlen = len - offset;

	s.set(msg + offset, nlen);

	return s;
}

String String::split(char delimiter)
{
	for (Uint32 i = 0; i < len; ++i)
		if (msg[i] == delimiter)
		{
			String save = mid(i + 1).trim();

			String ret = left(i).trim();

			set(save.msg, save.len);

			return ret;
		}

	return trim();
}

String String::trim()
{
	String s;

	Uint32 i;

	for (i = 0; i < len; ++i)
		if (msg[i] != ' ')
		{
			s = mid(i, len - i);
			break;
		}

	for (i = s.len - 1; i >= 0; --i)
		if (s[i] != ' ')
		{
			s = s.left(i + 1);
			break;
		}

	return s;
}

char *spaces = "                                                                                                              ";

String String::lfill(Uint32 nlen)
{
	if (len >= nlen)
		return right(nlen);

	String s;
	s.set(spaces, nlen - len);
	s.append(msg, len);

	return s;
}

String String::rfill(Uint32 nlen)
{
	if (len >= nlen)
		return left(nlen);

	String s(*this);
	s.append(spaces, nlen - len);

	return s;
}

void String::lcase()
{
	tolower(msg);
}

void String::ucase()
{
	toupper(msg);
}

int String::toInteger()
{
	return getInteger(msg, 10);
}

int String::toInteger(int base)
{
	return getInteger(msg, base);
}


//////// Vectors ////////

Vector::Vector()
{
	x = 0;
	y = 0;
}

Vector::Vector(Vector &v)
{
	x = v.x;
	y = v.y;
}

Vector::Vector(Sint32 nx, Sint32 ny)
{
	x = nx;
	y = ny;
}

bool Vector::operator==(Vector &v)
{
	return x == v.x && y == v.y;
}

bool Vector::operator!=(Vector &v)
{
	return x != v.x || y != v.y;
}

void Vector::operator=(Vector &v)
{
	x = v.x;
	y = v.y;
}

void Vector::operator=(Sint32 n)
{
	x = n;
	y = n;
}

Vector Vector::operator+(Vector &v)
{
	Vector a(x + v.x, y + v.y);
	return a;
}

void Vector::operator+=(Vector &v)
{
	x += v.x;
	y += v.y;
}

Vector Vector::operator-(Vector &v)
{
	Vector a(x - v.x, y - v.y);
	return a;
}

void Vector::operator-=(Vector &v)
{
	x -= v.x;
	y -= v.y;
}

Vector Vector::operator*(Vector &v)
{
	Vector a(x * v.x, y * v.y);
	return a;
}

void Vector::operator*=(Vector &v)
{
	x *= v.x;
	y *= v.y;
}

Vector Vector::operator/(Vector &v)
{
	Vector a(x / v.x, y / v.y);
	return a;
}

void Vector::operator/=(Vector &v)
{
	x /= v.x;
	y /= v.y;
}

Vector Vector::operator+(Sint32 n)
{
	Vector a(x + n, y + n);
	return a;
}

void Vector::operator+=(Sint32 n)
{
	x += n;
	y += n;
}

Vector Vector::operator-(Sint32 n)
{
	Vector a(x - n, y - n);
	return a;
}

void Vector::operator-=(Sint32 n)
{
	x -= n;
	y -= n;
}

Vector Vector::operator*(Sint32 n)
{
	Vector a(x * n, y * n);
	return a;
}

void Vector::operator*=(Sint32 n)
{
	x *= n;
	y *= n;
}

Vector Vector::operator/(Sint32 n)
{
	if (n == 0) return *this;
	Vector a(x / n, y / n);
	return a;
}

void Vector::operator/=(Sint32 n)
{
	if (n == 0) return;
	x /= n;
	y /= n;
}

void Vector::set(Sint32 nx, Sint32 ny)
{
	x = nx;
	y = ny;
}

Sint32 Vector::distance(Vector &v)
{
	Sint32 a = (x - v.x),
		   b = (y - v.y),
		   c = a * a + b * b;

	return SQRT(c);
}
